<?php

/**
 * Internal toolset for Views remote pager module.
 */

class VRP {

  // Internal caching.
  private static $navigation, $pager;

  // Related to original view.
  private static $view, $nid, $position;

  /**
   * Adds a vrp query to the url of any node which is result of a VRP view.
   *
   * @param object $node
   *   A node object about to be linked.
   *
   * @return bool|array
   *   If VRP data was found in the query,
   */
  public static function getViewParams($node = NULL) {

    $ret = FALSE;
    if (
      (is_object($node))
      &&
      ($view =@ $node->view)
      &&
      (is_object($view))
      &&
      // Check if the display is vrp-enabled.
      ($display =@ $view->display[$view->current_display])
      &&
      (!empty($display->display_options['vrp']))
      &&
      (!empty($display->display_options['vrp']['vrp_enabled']))
      &&
      // Check if this actually is a paged result.
      (isset($view->row_index))
      &&
      // Make sure a views ID is provided.
      ($source_view_id =@ $view->name)
      &&
      // Make sure a views ID is provided.
      ($source_display_id =@ $view->current_display)
    ) {
      // Access current view display:
      $display = $view->display[$source_display_id];
      // Prepare query parameters.
      $original_query = $_GET;
      unset($original_query['q']);
      $params = array(
        'v' => $source_view_id,
        'd' => $source_display_id,
      );
      if ($original_query = drupal_http_build_query($original_query)) {
        $params['q'] = $original_query;
      }
      // Add path parameter for return in case the parent views display
      // does NOT provide a "path" property:
      if (!isset($display->display_options) || !isset($display->display_options['path'])) {
        $params['p'] = current_path();
      }
      // Track arguments.
      if ($arguments =@ $view->args) {
        $params['a'] = $arguments;
      }
      // Add views internal params. These may not live in the query,
      // but reflect the default settings for exposed filters.
      if (($views_query =@ $view->exposed_raw_input) && is_array($views_query)) {
        $params['vq'] = urlencode(drupal_http_build_query($views_query));
      }

      $ret = array('vrp' => $params);
    }
    return $ret;
  }

  /**
   * Builds a navigation from the current global request, if it indicates so.
   *
   * Checks the request data for existing views data. If such data is found,
   * the corresponding view is looked up, executed and the position of the
   * current node in the result set is evaluated.
   *
   * @param bool $force
   *   Rebuild, even if previously cached.
   *
   * @return array|bool
   *
   */
  public static function getNavigationData($force = FALSE) {

    if (!isset(self::$navigation) || $force) {
      $ret = FALSE;
      $originalQuery = $_GET;
      // Remove original path request.
      unset($originalQuery['q']);

      $q = explode('/', current_path());
      // Will currently only work on node pages.
      if (is_array($q) && array_shift($q) == 'node') {
        if ($nid = array_shift($q)) {
          self::$nid = $nid;
          if ($query =@ $originalQuery['vrp']) {
            if (($view_id =@ $query['v']) && ($display_id =@ $query['d'])){
              if ($view =@ views_get_view($view_id)) {
                if ($view->set_display($display_id)) {
                  // Make views display accessible.
                  $currentDisplay = $view->display[$display_id];
                  // Remember original pager limit.
                  $originalLimit = 0;
                  if ($originalPager =@ $currentDisplay->display_options['pager']) {
                    $originalLimit = (int) $originalPager['options']['items_per_page'];
                  }
                  if ($args =@ $query['a']) {
                    $view->set_arguments($args);
                  }
                  if ($exposed =@ $query['vq']) {
                    if ($exposed = drupal_get_query_array(urldecode($exposed))) {
                      $view->set_exposed_input($exposed);
                    }
                  }
                  // Alter view parameters to get all items.
                  $view->set_items_per_page(0);
                  $view->set_offset(0);
                  // Execute updated view to reconstruct actual paging data.
                  $view->build();
                  $view->pre_execute();
                  $view->execute();
                  if (!empty($view->result)) {
                    // Identify node position in the view.
                    foreach ($view->result as $pos => $data) {
                      if ($data->nid == self::$nid) {
                        // Will ignore non-distinct queries by design, so multiple positions
                        // will not work.
                        $currentPosition = $pos;
                        break;
                      }
                    }
                    $currentPage = $originalLimit > 0 ? floor($currentPosition / $originalLimit) : 0;
                    $max = count($view->result) - 1;

                    // Prepare result array:
                    $ret = array();
                    if ((!$options =@ $currentDisplay->display_options) || (!$parentPath =@ $options['path'])) {
                      $parentPath =@ $query['p'];
                    }
                    else {
                      // Remove argument placeholders.
                      $parentPath = str_replace('/%', '', $parentPath);
                    }
                    if (!empty($parentPath)) {
                      if ($arguments =@ $query['a']) {
                        array_unshift($arguments, $parentPath);
                        $parentPath = implode('/', $arguments);
                      }
                      $ret['up'] = $parentPath;
                    }
                    if ($currentPosition > 0) {
                      $ret += array(
                        'first' => 0,
                        'previous' => $currentPosition - 1,
                      );
                    }

                    if ($currentPosition < $max) {
                      $ret += array(
                        'next' => $currentPosition + 1,
                        'last' => $max,
                      );
                    }

                    // Build navigation links; respect original query.
                    foreach (array_keys($ret) as $key) {
                      // Create actual path except for "up" (already done):
                      $newQuery = $originalQuery;
                      if ($key == 'up') {
                        $path = $ret[$key];
                        if ($currentPage) {
                          $newQuery['page'] = $currentPage;
                        }
                        if (!empty($exposed)) {
                          $newQuery += $exposed;
                        }
                        unset($newQuery['vrp']); // Not needed in the parent view!
                        $fragment = 'node-' . self::$nid;
                      }
                      else {
                        // Determine node nid from views result
                        // (is always present in "content" row style).
                        $pos = $ret[$key];
                        $nid = $view->result[$pos]->nid;
                        $path = "node/$nid";
                        $fragment = NULL;
                      }
                      $ret[$key] = array(
                        'href' => url($path, array(
                          'query' => $newQuery,
                          'fragment' => $fragment,
                        )),
                        'path' => $path,
                        'query' => $newQuery,
                        'fragment' => $fragment,
                      );
                    };
                  }
                }
              }
            }
          }
        }
      }
      self::$navigation = $ret;
    }
    return self::$navigation;
  }

  /**
   * Adds "<link rel=...>" tags depending on the actual navigation state.
   */
  public static function injectRelations() {
    if ($nav = self::getNavigationData()) {
      foreach ($nav as $type => $data) {
        drupal_add_html_head(array(
          '#tag' => 'link',
          '#attributes' => array( // Set up an array of attributes inside the tag
            'rel' => $type,
            'href' => $data['href'],
          ),
        ), "vrp_nav_$type");
      }
    }
  }

  /**
   * Builds a render array similar to core pagers.
   *
   * @return array
   *   A pager render array (may be empty if there is nothing to page).
   */
  public static function getPager() {
    if (!isset(self::$pager)) {
      $pagerDefaults = array(
        'first' => t('First'),
        'previous' => t('Previous'),
        'up' => t('Up'),
        'next' => t('Next'),
        'last' => t('Last'),
      );
      $ret = array();
      // Initialize internal views data and check if there is something to page.
      if ($navData = self::getNavigationData()) {
        $listItems = array();
        foreach ($pagerDefaults as $key => $title) {
          $label = '<span class="text">' . check_plain($title) . '</span>';
          if (!empty($navData[$key])) {
            // Add a link.
            $element = l($label, $navData[$key]['path'], array(
              'query' => $navData[$key]['query'],
              'fragment' => $navData[$key]['fragment'],
              'html' => TRUE,
              'attributes' => array(
                'title' => check_plain($title),
              ),
            ));
          }
          else {
            // Add plain text.
            $element = "<span class=\"inactive\">$label</span>";
          }
          $listItems[] = array(
            'data' => $element,
            'class' => array("pager-$key"),
          );
        }
        $ret = array(
          '#theme' => 'item_list',
          '#items' => $listItems,
          '#attributes' => array(
            'class' => array('vrp-pager', 'pager'),
          ),
        );
      }
      self::$pager = $ret;
    }
    return self::$pager;
  }
}
